"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const chart_1 = __importDefault(require("./chart"));
const image_1 = __importDefault(require("./image"));
const file_1 = __importDefault(require("./helper/file"));
const xml_1 = __importDefault(require("./helper/xml"));
class Slide {
    constructor(params) {
        this.sourceTemplate = params.template;
        this.sourceNumber = params.number;
        this.sourcePath = `ppt/slides/slide${this.sourceNumber}.xml`;
        this.relsPath = `ppt/slides/_rels/slide${this.sourceNumber}.xml.rels`;
        this.modifications = [];
        this.appendElements = [];
        this.appendRelations = [];
    }
    modify(callback) {
        this.modifications.push(callback);
    }
    addElement(presName, slideNumber, selector, callback) {
        return __awaiter(this, void 0, void 0, function* () {
            let template = this.root.template(presName);
            let sourcePath = `ppt/slides/slide${slideNumber}.xml`;
            let sourceArchive = yield template.archive;
            let sourceElement = yield xml_1.default.findByElementName(sourceArchive, sourcePath, selector);
            if (!sourceElement) {
                throw new Error(`Can't find ${selector} on slide ${slideNumber} in ${presName}`);
            }
            let appendElement = sourceElement.cloneNode(true);
            this.analyzeElement(appendElement, sourceArchive);
            if (callback !== undefined) {
                if (callback instanceof Array) {
                    callback.forEach(cb => cb(appendElement));
                }
                else {
                    callback(appendElement);
                }
            }
            this.appendElements.push(appendElement);
            return this;
        });
    }
    setTarget(targetTemplate) {
        return __awaiter(this, void 0, void 0, function* () {
            this.targetTemplate = targetTemplate;
            this.targetArchive = yield targetTemplate.archive;
            this.targetNumber = targetTemplate.count('slides');
            this.targetPath = `ppt/slides/slide${this.targetNumber}.xml`;
            this.targetRelsPath = `ppt/slides/_rels/slide${this.targetNumber}.xml.rels`;
        });
    }
    append() {
        return __awaiter(this, void 0, void 0, function* () {
            this.sourceArchive = yield this.sourceTemplate.archive;
            yield this.copySlideFiles();
            yield this.copyRelatedContent();
            yield this.addSlideToPresentation();
            if (this.appendRelations.length) {
                yield this.appendImportedRelations();
            }
            if (this.appendElements.length) {
                yield this.appendImportedElements();
            }
            yield this.applyModifications();
        });
    }
    analyzeElement(appendElement, sourceArchive) {
        return __awaiter(this, void 0, void 0, function* () {
            let isChart = appendElement.getElementsByTagName('c:chart');
            if (isChart.length) {
                let sourceRid = isChart[0].getAttribute('r:id');
                let chartRels = yield xml_1.default.getTargetsFromRelationships(sourceArchive, this.relsPath, '../charts/chart');
                this.appendRelations.push({
                    relation: chartRels.find(rel => rel.rId === sourceRid),
                    sourceArchive: sourceArchive
                });
            }
        });
    }
    appendImportedElements() {
        return __awaiter(this, void 0, void 0, function* () {
            let slideXml = yield xml_1.default.getXmlFromArchive(this.targetArchive, this.targetPath);
            let tree = slideXml.getElementsByTagName('p:spTree')[0];
            this.appendElements.forEach(element => {
                tree.appendChild(element);
            });
            yield xml_1.default.writeXmlToArchive(this.targetArchive, this.targetPath, slideXml);
        });
    }
    appendImportedRelations() {
        return __awaiter(this, void 0, void 0, function* () {
            for (let i in this.appendRelations) {
                let relation = this.appendRelations[i].relation;
                let sourceArchive = this.appendRelations[i].sourceArchive;
                let newChart = new chart_1.default(relation, sourceArchive, this.targetNumber, true);
                this.targetTemplate.incrementCounter('charts');
                yield this.targetTemplate.appendChart(newChart);
                this.modifications.push(chart => {
                    chart.getElementsByTagName('c:chart')[0].setAttribute('r:id', newChart.createdRid);
                });
            }
        });
    }
    applyModifications() {
        return __awaiter(this, void 0, void 0, function* () {
            for (let m in this.modifications) {
                let xml = yield xml_1.default.getXmlFromArchive(this.targetArchive, this.targetPath);
                this.modifications[m](xml);
                yield xml_1.default.writeXmlToArchive(this.targetArchive, this.targetPath, xml);
            }
        });
    }
    copySlideFiles() {
        return __awaiter(this, void 0, void 0, function* () {
            file_1.default.zipCopy(this.sourceArchive, `ppt/slides/slide${this.sourceNumber}.xml`, this.targetArchive, `ppt/slides/slide${this.targetNumber}.xml`);
            file_1.default.zipCopy(this.sourceArchive, `ppt/slides/_rels/slide${this.sourceNumber}.xml.rels`, this.targetArchive, `ppt/slides/_rels/slide${this.targetNumber}.xml.rels`);
        });
    }
    addSlideToPresentation() {
        return __awaiter(this, void 0, void 0, function* () {
            let relId = yield xml_1.default.getNextRelId(this.targetArchive, 'ppt/_rels/presentation.xml.rels');
            yield this.appendToSlideRel(this.targetArchive, relId, this.targetNumber),
                yield this.appendToSlideList(this.targetArchive, relId),
                yield this.appendToContentType(this.targetArchive, this.targetNumber);
        });
    }
    appendToSlideRel(rootArchive, relId, slideCount) {
        return xml_1.default.append({
            archive: rootArchive,
            file: `ppt/_rels/presentation.xml.rels`,
            parent: (xml) => xml.getElementsByTagName('Relationships')[0],
            tag: 'Relationship',
            attributes: {
                Id: relId,
                Type: `http://schemas.openxmlformats.org/officeDocument/2006/relationships/slide`,
                Target: `slides/slide${slideCount}.xml`
            }
        });
    }
    appendToSlideList(rootArchive, relId) {
        return xml_1.default.append({
            archive: rootArchive,
            file: `ppt/presentation.xml`,
            parent: (xml) => xml.getElementsByTagName('p:sldIdLst')[0],
            tag: 'p:sldId',
            attributes: {
                id: (xml) => xml_1.default.getMaxId(xml.getElementsByTagName('p:sldId'), 'id', true),
                'r:id': relId
            }
        });
    }
    appendToContentType(rootArchive, slideCount) {
        return xml_1.default.append(xml_1.default.createContentTypeChild(rootArchive, {
            PartName: `/ppt/slides/slide${slideCount}.xml`,
            ContentType: `application/vnd.openxmlformats-officedocument.presentationml.slide+xml`
        }));
    }
    copyRelatedContent() {
        return __awaiter(this, void 0, void 0, function* () {
            let charts = yield xml_1.default.getTargetsFromRelationships(this.sourceArchive, this.relsPath, '../charts/chart');
            for (let i in charts) {
                let newChart = new chart_1.default(charts[i], this.sourceArchive, this.targetNumber);
                this.targetTemplate.incrementCounter('charts');
                yield this.targetTemplate.appendChart(newChart);
            }
            let images = yield xml_1.default.getTargetsByRelationshipType(this.sourceArchive, this.relsPath, "http://schemas.openxmlformats.org/officeDocument/2006/relationships/image");
            for (let i in images) {
                let newImage = new image_1.default(images[i], this.sourceArchive, this.targetNumber);
                this.targetTemplate.incrementCounter('images');
                yield this.targetTemplate.appendImage(newImage);
            }
        });
    }
}
exports.default = Slide;
//# sourceMappingURL=slide.js.map